/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.readers.its.jira;

// REMPL API from com.rempl:rempl-api from com.rempl:rempl-api
import com.rempl.api.Logger;

// Bridge API from com.rempl.readers.its:bridge-its-api
import com.rempl.readers.its.bridge.Bridge;
import com.rempl.readers.its.bridge.BridgeException;
import com.rempl.readers.its.bridge.Issue;

// bridge utils from com.rempl.readers:readers-bridge-utils
import com.rempl.readers.utils.RpcAdapter;

// for manipulation of issues
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

// for string manipulations
import org.apache.commons.lang.StringUtils;

/**
 * Bridge between JIRA and REMPL Reader.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: XmlRpcBridge.java 1389 2011-05-23 23:07:59Z guard $
 */
public final class XmlRpcBridge implements Bridge {

    /**
     * Parameter name to use for setting URL of JIRA.
     */
    public static final String KEY_URL = "url";

    /**
     * Parameter name to set username in JIRA.
     */
    public static final String KEY_USER = "user";

    /**
     * Parameter name to set JIRA password.
     */
    public static final String KEY_PASSWORD = "password";

    /**
     * JIRA filter to use to get all issues.
     */
    public static final String KEY_FILTER = "filter";

    /**
     * Parameter name to use for setting factory.
     */
    private static final String KEY_FACTORY = "transportFactory";

    /**
     * Bridge to JIRA.
     */
    private JiraServices services;

    /**
     * JIRA filter ID.
     */
    private String filter;

    /**
     * Public ctor.
     */
    public XmlRpcBridge() {
        this.services = null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void configure(final Map<String, String> params)
        throws BridgeException {
        this.services = this.connect(params);
        final Map<String, String> info =
            (Map<String, String>) this.services.call("getServerInfo");
        Logger.info(
            this,
            "JIRA found at %s (version %s)",
            info.get("baseUrl"),
            info.get("version")
        );
        final Object[] filters = (Object[])
            this.services.call("getFavouriteFilters");
        if (!params.containsKey(this.KEY_FILTER)) {
            throw new BridgeException(
                "JIRA filter name has to be set with parameter \""
                + this.KEY_FILTER
                + "\""
            );
        }
        final Collection<String> names = new ArrayList<String>();
        for (Object ftr : filters) {
            final Map details = (Map) ftr;
            final String name = (String) details.get("name");
            names.add(name);
            if (name.equals(params.get(this.KEY_FILTER))) {
                this.filter = (String) details.get("id");
            }
        }
        if (this.filter == null) {
            throw new BridgeException(
                "JIRA filter \""
                + params.get(this.KEY_FILTER)
                + "\" not found among: "
                + StringUtils.join(names, ", ")
            );
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Map<String, Issue> retrieve() throws BridgeException {
        if (this.services == null) {
            throw new IllegalStateException("Use configure() first");
        }
        final Map<String, Issue> issues = new HashMap<String, Issue>();
        final Object[] tickets = (Object[]) this.services.call(
            "getIssuesFromFilter",
            this.filter
        );
        for (Object ticket : tickets) {
            final Map details = (Map) ticket;
            issues.put(
                (String) details.get("key"),
                new JiraIssue(this.services, details)
            );
        }
        Logger.info(
            this,
            "#retrieve(): %d tickets loaded from JIRA",
            issues.size()
        );
        return issues;
    }

    /**
     * Connect and return JiraServices.
     * @param params Arguments for connectino
     * @return Created connection
     * @throws BridgeException If something goes wrong
     * @see #configure(Map)
     */
    private JiraServices connect(final Map<String, String> params)
        throws BridgeException {
        if (!params.containsKey(this.KEY_URL)) {
            throw new BridgeException(
                "Configuration parameter \""
                + this.KEY_URL
                + "\" is mandatory"
            );
        }
        final RpcAdapter rpc =
            new RpcAdapter(params.get(this.KEY_URL), "/rpc/xmlrpc");
        if (params.containsKey(this.KEY_FACTORY)) {
            rpc.setTransportFactory(params.get(this.KEY_FACTORY));
        }
        if (params.containsKey(this.KEY_USER)) {
            return new JiraServices(
                rpc,
                params.get(this.KEY_USER),
                params.get(this.KEY_PASSWORD)
            );
        }
        return new JiraServices(rpc);
    }

}
